# -*- coding: utf-8 -*-
'''Test WGC surface intercpt point calculation.'''

import pytest

from webgeocalc import SurfaceInterceptPoint
from webgeocalc.errors import CalculationInvalidAttr, CalculationUndefinedAttr

@pytest.fixture
def kernels():
    '''Cassini kernel set.'''
    return 5

@pytest.fixture
def time():
    '''Input time.'''
    return '2012-10-19T08:24:00.000'

@pytest.fixture
def target():
    '''Input target.'''
    return 'ENCELADUS'

@pytest.fixture
def target_frame():
    '''Input target frame.'''
    return 'IAU_ENCELADUS'

@pytest.fixture
def observer():
    '''Input observer.'''
    return 'CASSINI'

@pytest.fixture
def intercept_vector_type_1():
    '''Input intercept vector type 1.'''
    return 'INSTRUMENT_BORESIGHT'

@pytest.fixture
def intercept_vector_type_2():
    '''Input intercept vector type 2.'''
    return 'REFERENCE_FRAME_AXIS'

@pytest.fixture
def intercept_vector_type_3():
    '''Input intercept vector type 3.'''
    return 'VECTOR_IN_INSTRUMENT_FOV'

@pytest.fixture
def intercept_vector_type_4():
    '''Input intercept vector type 4.'''
    return 'VECTOR_IN_REFERENCE_FRAME'

@pytest.fixture
def intercept_instrument():
    '''Input intercept instrument.'''
    return 'CASSINI_ISS_NAC'

@pytest.fixture
def intercept_frame():
    '''Input intercept frame.'''
    return 'CASSINI_ISS_NAC'

@pytest.fixture
def intercept_frame_axis():
    '''Input intercept axis.'''
    return 'Z'

@pytest.fixture
def corr():
    '''Input aberration correction.'''
    return 'NONE'

@pytest.fixture
def state():
    '''Input state.'''
    return 'LATITUDINAL'

@pytest.fixture
def params_1(kernels, time, target, target_frame, observer, intercept_vector_type_1,
             intercept_instrument, corr, state):
    '''Input parameters from WGC API example 1.'''
    return {
        'kernels': kernels,
        'times': time,
        'target': target,
        'target_frame': target_frame,
        'observer': observer,
        'intercept_vector_type': intercept_vector_type_1,
        'intercept_instrument': intercept_instrument,
        'aberration_correction': corr,
        'state_representation': state,
    }

@pytest.fixture
def payload_1(kernels, time, target, target_frame, observer, intercept_vector_type_1,
              intercept_instrument, corr, state):
    '''Payload from WGC API example 1.'''
    return {
        "kernels": [{
            "type": "KERNEL_SET",
            "id": kernels,
        }],
        "timeSystem": "UTC",
        "timeFormat": "CALENDAR",
        "times": [
            time,
        ],
        "calculationType": "SURFACE_INTERCEPT_POINT",
        "target": target,
        "shape1": "ELLIPSOID",
        "targetFrame": target_frame,
        "observer": observer,
        "interceptVectorType": intercept_vector_type_1,
        "interceptInstrument": intercept_instrument,
        "aberrationCorrection": corr,
        "stateRepresentation": state,
    }

@pytest.fixture
def params_2(kernels, time, target, target_frame, observer, intercept_vector_type_2,
             intercept_frame, intercept_frame_axis, corr):
    '''Input parameters from WGC API example 2.'''
    return {
        'kernels': kernels,
        'times': time,
        'target': target,
        'target_frame': target_frame,
        'observer': observer,
        'intercept_vector_type': intercept_vector_type_2,
        'intercept_frame': intercept_frame,
        'intercept_frame_axis': intercept_frame_axis,
        'aberration_correction': corr,
    }

@pytest.fixture
def payload_2(kernels, time, target, target_frame, observer, intercept_vector_type_2,
              intercept_frame, intercept_frame_axis, corr):
    '''Payload from WGC API example 2.'''
    return {
        "kernels": [{
            "type": "KERNEL_SET",
            "id": kernels,
        }],
        "timeSystem": "UTC",
        "timeFormat": "CALENDAR",
        "times": [
            time,
        ],
        "calculationType": "SURFACE_INTERCEPT_POINT",
        "target": target,
        "shape1": "ELLIPSOID",
        "targetFrame": target_frame,
        "observer": observer,
        "interceptVectorType": intercept_vector_type_2,
        "interceptFrame": intercept_frame,
        "interceptFrameAxis": intercept_frame_axis,
        "aberrationCorrection": corr,
        "stateRepresentation": "RECTANGULAR",
    }

@pytest.fixture
def params_3(kernels, time, target, target_frame, observer, intercept_vector_type_3,
             intercept_instrument, corr):
    '''Input parameters from WGC API example 3.'''
    return {
        'kernels': kernels,
        'times': time,
        'target': target,
        'target_frame': target_frame,
        'observer': observer,
        'intercept_vector_type': intercept_vector_type_3,
        "intercept_instrument": intercept_instrument,
        'intercept_vector_x': 0,
        'intercept_vector_y': 0,
        'intercept_vector_z': 1,
        'aberration_correction': corr,
    }

@pytest.fixture
def payload_3(kernels, time, target, target_frame, observer, intercept_vector_type_3,
              intercept_instrument, corr):
    '''Payload from WGC API example 3.'''
    return {
        "kernels": [{
            "type": "KERNEL_SET",
            "id": kernels,
        }],
        "timeSystem": "UTC",
        "timeFormat": "CALENDAR",
        "times": [
            time,
        ],
        "calculationType": "SURFACE_INTERCEPT_POINT",
        "target": target,
        "shape1": "ELLIPSOID",
        "targetFrame": target_frame,
        "observer": observer,
        "interceptVectorType": intercept_vector_type_3,
        "interceptInstrument": intercept_instrument,
        "interceptVectorX": 0,
        "interceptVectorY": 0,
        "interceptVectorZ": 1,
        "aberrationCorrection": corr,
        "stateRepresentation": "RECTANGULAR",
    }

@pytest.fixture
def params_4(kernels, time, target, target_frame, observer, intercept_vector_type_4,
             intercept_frame, corr):
    '''Input parameters from WGC API example 4.'''
    return {
        'kernels': kernels,
        'times': time,
        'target': target,
        'target_frame': target_frame,
        'observer': observer,
        'intercept_vector_type': intercept_vector_type_4,
        "intercept_frame": intercept_frame,
        'intercept_vector_ra': 0,
        'intercept_vector_dec': 90,
        'aberration_correction': corr,
    }

@pytest.fixture
def payload_4(kernels, time, target, target_frame, observer, intercept_vector_type_4,
              intercept_frame, corr):
    '''Payload from WGC API example 4.'''
    return {
        "kernels": [{
            "type": "KERNEL_SET",
            "id": kernels,
        }],
        "timeSystem": "UTC",
        "timeFormat": "CALENDAR",
        "times": [
            time,
        ],
        "calculationType": "SURFACE_INTERCEPT_POINT",
        "target": target,
        "shape1": "ELLIPSOID",
        "targetFrame": target_frame,
        "observer": observer,
        "interceptVectorType": intercept_vector_type_4,
        "interceptFrame": intercept_frame,
        "interceptVectorRA": 0,
        "interceptVectorDec": 90,
        "aberrationCorrection": corr,
        "stateRepresentation": "RECTANGULAR",
    }


def test_surface_intercept_point_payload_boresight(params_1, payload_1):
    '''Test surface intercept point payload with INSTRUMENT_BORESIGHT.'''
    assert SurfaceInterceptPoint(**params_1).payload == payload_1

def test_surface_intercept_point_payload_reference_frame_axis(params_2, payload_2):
    '''Test surface intercept point payload with REFERENCE_FRAME_AXIS.'''
    assert SurfaceInterceptPoint(**params_2).payload == payload_2

def test_surface_intercept_point_payload_vector_in_instrument_fov(params_3, payload_3):
    '''Test surface intercept point payload with VECTOR_IN_INSTRUMENT_FOV.'''
    assert SurfaceInterceptPoint(**params_3).payload == payload_3

def test_surface_intercept_point_payload_vector_in_reference_frame(params_4, payload_4):
    '''Test surface intercept point payload with VECTOR_IN_REFERENCE_FRAME.'''
    assert SurfaceInterceptPoint(**params_4).payload == payload_4

def test_surface_intercept_point_attr_error(params_1):
    '''Test errors when surface intercept point attributes is invalid.'''
    with pytest.raises(CalculationInvalidAttr):
        SurfaceInterceptPoint(shape_1='POINT', **params_1)

def test_surface_intercept_point_payload_vector_in_instrument_fov_err(params_3):
    '''Test errors when VECTOR_IN_INSTRUMENT_FOV is invalid.'''
    del params_3['intercept_vector_x']
    with pytest.raises(CalculationUndefinedAttr):
        # Missing 'intercept_vector_x'
        SurfaceInterceptPoint(**params_3)



# -*- coding: utf-8 -*-
'''Test WGC angular size calculation.'''

import pytest

from webgeocalc import OsculatingElements

@pytest.fixture
def kernels():
    '''Kernels sets Solar and Cassini.'''
    return [1, 5]

@pytest.fixture
def time():
    '''Input time.'''
    return '2012-10-19T08:24:00.000'

@pytest.fixture
def orbiting():
    '''Orbiting body.'''
    return 'CASSINI'

@pytest.fixture
def center():
    '''Center body.'''
    return 'SATURN'


@pytest.fixture
def params(kernels, time, orbiting, center):
    '''Input parameters from WGC API example.'''
    return {
        'kernels': kernels,
        'times': time,
        'orbiting_body': orbiting,
        'center_body': center,
    }

@pytest.fixture
def payload(kernels, time, orbiting, center):
    '''Payload from WGC API example.'''
    return {
        "kernels": [{
            "type": "KERNEL_SET",
            "id": kernels[0],
        }, {
            "type": "KERNEL_SET",
            "id": kernels[1],
        }],
        "timeSystem": "UTC",
        "timeFormat": "CALENDAR",
        "times": [
            time,
        ],
        "calculationType": "OSCULATING_ELEMENTS",
        "orbitingBody": orbiting,
        "centerBody": center,
        "referenceFrame": "J2000",
    }

def test_osculating_elements_payload(params, payload):
    '''Test osculating elements payload.'''
    assert OsculatingElements(**params).payload == payload



# -*- coding: utf-8 -*-
'''WebGeoCalc columns and results types.'''

from .errors import ResultAttributeError, ResultTypeError

def get_type(classname):
    '''Get type based on classname.'''
    if classname not in TYPES.keys():
        raise ResultTypeError(classname)
    return TYPES[classname]

class ColumnResult(object):
    '''Column result generic object.'''

    def __init__(self, json):
        self._json = json

    def __str__(self):
        return self.name

    def __repr__(self):
        return f"<{self.__class__.__name__}> {str(self)}"

    def __getattr__(self, attr):
        if attr not in self._json.keys():
            raise ResultAttributeError(self, attr)
        return self._json[attr]

    def keys(self):
        '''JSON keys.'''
        return self._json.keys()

    def values(self):
        '''JSON values.'''
        return self._json.values()

    def items(self):
        '''JSON items.'''
        return self._json.items()

class ResultType(ColumnResult):
    '''Result Type object for item generic interface.'''

    def __init__(self, json):
        super().__init__(json)

    def __int__(self):
        return int(self.id)

    def __repr__(self):
        return f"<{self.__class__.__name__}> {str(self)} (id: {int(self)})"

    def __contains__(self, item):
        '''Check if items is equal to int or str.'''
        if isinstance(item, int):
            return item == int(self)
        return item.lower() in str(self).lower()

class KernelSetDetails(ResultType):
    '''Kernel set details.'''

    def __init__(self, json):
        super().__init__(json)

    def __int__(self):
        return int(self.kernelSetId)

    def __str__(self):
        return self.caption

class BodyData(ResultType):
    '''Body data.'''

    def __init__(self, json):
        super().__init__(json)

class FrameData(ResultType):
    '''Frame data.'''

    def __init__(self, json):
        super().__init__(json)

class InstrumentData(ResultType):
    '''Instrument data.'''

    def __init__(self, json):
        super().__init__(json)


TYPES = globals()



# -*- coding: utf-8 -*-
'''Configuration file for the Sphinx documentation builder.

This file does only contain a selection of the most common options. For a
full list see the documentation:
http://www.sphinx-doc.org/en/master/config
'''

# -- Project information -----------------------------------------------------

project = 'WebGeoCalc API'
copyright = '2019, Benoit Seignovert'
author = 'Benoit Seignovert'

# -- General configuration ---------------------------------------------------

extensions = [
    'pbr.sphinxext',
    'sphinx.ext.autodoc',
    'sphinx.ext.napoleon',
    'sphinx.ext.doctest',
    'sphinx.ext.coverage',
    'sphinx.ext.viewcode',
]

source_suffix = '.rst'
master_doc = 'index'
language = None
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']

# -- Options for HTML output -------------------------------------------------

html_theme = 'sphinx_rtd_theme'
htmlhelp_basename = 'WebGeoCalcAPIdoc'

# -- Extension configuration -------------------------------------------------
autodoc_default_flags = ['members', 'show-inheritance']
autodoc_member_order = 'bysource'
pygments_style = None



# -*- coding: utf-8 -*-
'''Command line interface.'''

import argparse

from .api import API
from .errors import KernelSetNotFound, TooManyKernelSets

def cli_kernel_sets(argv=None):
    '''Get list of kernel sets available on the server.

    GET: /kernel-sets
    '''
    parser = argparse.ArgumentParser(
        description='List and search kernel sets available in WebGeocalc API.')
    parser.add_argument('--all', '-a', action='store_true',
                        help='List all kernel sets available')
    parser.add_argument('--kernel', '-k', metavar='NAME|ID', nargs='+',
                        help='Search a specific kernel by name or id')
    args, others = parser.parse_known_args(argv)

    if args.all:
        kernels = API.kernel_sets()
        print('\n'.join([f" - {str(kernel)}: (id: {int(kernel)})" for kernel in kernels]))

    elif args.kernel:
        for kernel in args.kernel:
            try:
                kernel = int(kernel)
            except ValueError:
                pass

            try:
                kernel = API.kernel_set(kernel)
                print(f" - {str(kernel)}: (id: {int(kernel)})")

            except (KernelSetNotFound, TooManyKernelSets) as err:
                print(err)
    else:
        parser.print_help()


def cli_bodies(argv=None):
    '''Get list of bodies available in a kernel set.

    GET: /kernel-set/{kernelSetId}/bodies
    '''
    parser = argparse.ArgumentParser(
        description='List bodies available in WebGeocalc API for a specific kernel set.')
    parser.add_argument('kernel', nargs='?', help='Kernel set name or id')
    parser.add_argument('--name', '-n', metavar='BODY', nargs='+',
                        help='Search a specific body by name or id')

    args, others = parser.parse_known_args(argv)

    if args.kernel:
        try:
            kernel = int(args.kernel)
        except ValueError:
            kernel = args.kernel

        bodies = API.bodies(kernel)

        if args.name:
            for body in bodies:
                for name in args.name:
                    if name in body:
                        print(f" - {str(body)}: (id: {int(body)})")
        else:
            print('\n'.join([f" - {str(body)}: (id: {int(body)})" for body in bodies]))
    else:
        parser.print_help()

def cli_frames(argv=None):
    '''Get list of frames available in a kernel set.

    GET: /kernel-set/{kernelSetId}/frames
    '''
    parser = argparse.ArgumentParser(
        description='List frames available in WebGeocalc API for a specific kernel set.')
    parser.add_argument('kernel', nargs='?', help='Kernel set name or id')
    parser.add_argument('--name', '-n', metavar='FRAME',
                        nargs='+', help='Search a specific frame by name or id')

    args, others = parser.parse_known_args(argv)

    if args.kernel:
        try:
            kernel = int(args.kernel)
        except ValueError:
            kernel = args.kernel

        frames = API.frames(kernel)

        if args.name:
            for frame in frames:
                for name in args.name:
                    if name in frame:
                        print(f" - {str(frame)}: (id: {int(frame)})")
        else:
            print('\n'.join([f" - {str(frame)}: (id: {int(frame)})" for frame in frames]))
    else:
        parser.print_help()

def cli_instruments(argv=None):
    '''Get list of instruments available in a kernel set.

    GET: /kernel-set/{kernelSetId}/instruments
    '''
    parser = argparse.ArgumentParser(description='List instruments available'
                                     'in WebGeocalc API for a specific kernel set.')
    parser.add_argument('kernel', nargs='?', help='Kernel set name or id')
    parser.add_argument('--name', '-n', metavar='INSTRUMENT', nargs='+',
                        help='Search a specific instrument by name or id')

    args, others = parser.parse_known_args(argv)

    if args.kernel:
        try:
            kernel = int(args.kernel)
        except ValueError:
            kernel = args.kernel

        instruments = API.instruments(kernel)

        if args.name:
            for instrument in instruments:
                for name in args.name:
                    if name in instrument:
                        print(f" - {str(instrument)}: (id: {int(instrument)})")
        else:
            print('\n'.join(
                [f" - {str(instrument)}: (id: {int(instrument)})"
                 for instrument in instruments]))
    else:
        parser.print_help()



# -*- coding: utf-8 -*-
'''Test WGC angular size calculation.'''

import pytest

from webgeocalc import AngularSize

@pytest.fixture
def kernels():
    '''Input kernel.'''
    return 5  # Cassini-Huygens

@pytest.fixture
def time():
    '''Input time.'''
    return '2012-10-19T08:24:00.000'

@pytest.fixture
def target():
    '''Input target name.'''
    return 'ENCELADUS'

@pytest.fixture
def observer():
    '''Input obserber name.'''
    return 'CASSINI'

@pytest.fixture
def corr():
    '''Input aberration correction.'''
    return 'CN+S'

@pytest.fixture
def params(kernels, time, target, observer, corr):
    '''Input parameters from WGC API example.'''
    return {
        'kernels': kernels,
        'times': time,
        'target': target,
        'observer': observer,
        'aberration_correction': corr,
    }

@pytest.fixture
def payload(kernels, time, target, observer, corr):
    '''Payload from WGC API example.'''
    return {
        "kernels": [{
            "type": "KERNEL_SET",
            "id": kernels,
        }],
        "timeSystem": "UTC",
        "timeFormat": "CALENDAR",
        "times": [
            time,
        ],
        "calculationType": "ANGULAR_SIZE",
        "target": target,
        "observer": observer,
        "aberrationCorrection": corr
    }

def test_angular_size_payload(params, payload):
    '''Test angular size payload.'''
    assert AngularSize(**params).payload == payload



# -*- coding: utf-8 -*-
'''Test WGC type results.'''
import pytest

from webgeocalc.types import ResultTypeError, get_type

def test_type_err():
    '''Test error unknown type results.'''
    with pytest.raises(ResultTypeError):
        get_type('WRONG_TYPE')



# -*- coding: utf-8 -*-
'''Test WGC API base urls.'''

import pytest

from requests import HTTPError

from webgeocalc import API
from webgeocalc.errors import APIError, APIReponseError, KernelSetNotFound, \
    ResultAttributeError, TooManyKernelSets
from webgeocalc.vars import API_URL

@pytest.fixture
def solar_system_kernel_set():
    '''Solar kernels API output.'''
    return {
        'caption': 'Solar System Kernels',
        'sclkId': '0',
        'description': 'Generic kernels for planets, satellites, and some asteroids '
                       'covering from 1950-01-01 to 2050-01-01.',
        'kernelSetId': '1',
        'missionId': 'gen',
    }

@pytest.fixture
def cassini_kernel_set():
    '''Cassini kernel API output.'''
    return {
        'caption': 'Cassini Huygens',
        'sclkId': '-82',
        'description': 'rchived CASSINI kernels covering from 1997-10-15 to 2017-09-15.',
        'kernelSetId': '5',
        'missionId': 'cassini',
    }

@pytest.fixture
def cassini_body():
    '''Cassini body API output.'''
    return {
        'id': -82,
        'name': 'CASSINI',
    }

@pytest.fixture
def cassini_frame():
    '''Cassini frame API output.'''
    return {
        'id': -82905,
        'name': 'CASSINI_KSO',
        'centerBodyID': 699,
        'frameClass': 5,
    }

@pytest.fixture
def cassini_instrument():
    '''CIRS instrument API output.'''
    return {
        'id': -82898,
        'name': 'CASSINI_CIRS_RAD',
    }

@pytest.fixture
def api_error():
    '''Error API output.'''
    return {
        "status": "ERROR",
        "message": None,
        "calculationId": "0788aba2-d4e5-4028-9ef1-4867ad5385e0"
    }

@pytest.fixture
def api_empty_data_response():
    '''Empty data API response.'''
    return {
        "status": "OK",
        "message": "The operation was successful.",
        "calculationId": "0788aba2-d4e5-4028-9ef1-4867ad5385e0",
    }

def test_api_default_url():
    '''Test default API url.'''
    assert API.url == API_URL

def test_response_err():
    '''Test GET and POST on invalid URLs.'''
    with pytest.raises(HTTPError):
        API.get('/')  # 404 error

    with pytest.raises(HTTPError):
        API.post('/', payload={})  # 404 error

def test_kernel_sets(solar_system_kernel_set):
    '''Test GET kernel sets from API.'''
    kernel_set = API.kernel_sets()[0]

    assert int(kernel_set) == int(solar_system_kernel_set['kernelSetId'])
    assert str(kernel_set) == solar_system_kernel_set['caption']

    for key in kernel_set.keys():
        assert key in solar_system_kernel_set.keys()

    for value in kernel_set.values():
        assert value in solar_system_kernel_set.values()

    for key, values in kernel_set.items():
        assert key in solar_system_kernel_set.keys()
        assert value in solar_system_kernel_set.values()

    with pytest.raises(ResultAttributeError):
        kernel_set.wrong_attr

def test_kernel_set_by_id():
    '''Test kernel set id.'''
    assert int(API.kernel_set(1)) == 1

def test_kernel_set_by_full_name():
    '''Test kernel set name.'''
    assert str(API.kernel_set('Solar System Kernels')) == 'Solar System Kernels'

def test_kernel_set_by_name_not_case_sensitive():
    '''Test case sensitivity of kernel set search.'''
    assert str(API.kernel_set('solar system kernels')) == 'Solar System Kernels'

def test_kernel_set_by_name_partial():
    '''Test partial kernel set search.'''
    assert str(API.kernel_set('Solar')) == 'Solar System Kernels'

def test_kernel_set_too_many_found():
    '''Test error if too many kernel sets are found.'''
    with pytest.raises(TooManyKernelSets):
        assert str(API.kernel_set('Cassini'))

def test_kernel_set_not_found():
    '''Test error if kernel set not found.'''
    with pytest.raises(KernelSetNotFound):
        assert str(API.kernel_set('Missing kernel'))


def test_kernel_set_id_for_str(cassini_kernel_set):
    '''Test kernel set search for id and error cases.'''
    kernel_set_id = int(cassini_kernel_set['kernelSetId'])
    kernel_set_caption = cassini_kernel_set['caption']

    assert API.kernel_set_id(kernel_set_caption) == kernel_set_id

    kernel_set = API.kernel_set(kernel_set_caption)
    assert API.kernel_set_id(kernel_set) == kernel_set_id

    with pytest.raises(TypeError):
        API.kernel_set_id(1.23)

def test_bodies(cassini_kernel_set, cassini_body):
    '''Test GET bodies from API.'''
    kernel_set_id = int(cassini_kernel_set['kernelSetId'])
    body = API.bodies(kernel_set_id)[0]

    assert int(body) == cassini_body['id']
    assert str(body) == cassini_body['name']

def test_frames(cassini_kernel_set, cassini_frame):
    '''Test GET frames from API.'''
    kernel_set_id = int(cassini_kernel_set['kernelSetId'])
    frame = API.frames(kernel_set_id)[58]

    assert int(frame) == cassini_frame['id']
    assert str(frame) == cassini_frame['name']

    for value in frame.values():
        assert value in cassini_frame.values()

def test_instruments(cassini_kernel_set, cassini_instrument):
    '''Test GET instruments from API.'''
    kernel_set_id = int(cassini_kernel_set['kernelSetId'])
    instrument = API.instruments(kernel_set_id)[0]

    assert int(instrument) == cassini_instrument['id']
    assert str(instrument) == cassini_instrument['name']

def test_api_read_error(api_error):
    '''Test error output from API.'''
    with pytest.raises(APIError):
        API.read(api_error)

def test_api_read_invalid(api_empty_data_response):
    '''Test error if response no valid data.'''
    with pytest.raises(APIReponseError):
        API.read(api_empty_data_response)



# -*- coding: utf-8 -*-
'''Test WGC calculation setup.'''

import pytest

from webgeocalc import Calculation
from webgeocalc.errors import CalculationConflictAttr, CalculationIncompatibleAttr, \
    CalculationInvalidAttr, CalculationRequiredAttr, CalculationUndefinedAttr

@pytest.fixture
def kernels():
    '''Input Solar System kernels set.'''
    return 1

@pytest.fixture
def calc():
    '''Input calculation type name.'''
    return 'STATE_VECTOR'

@pytest.fixture
def time():
    '''Input time.'''
    return '2012-10-19T08:24:00.000'

@pytest.fixture
def path():
    '''Input kernel path url.'''
    return 'https://path.to.server/kernel'

@pytest.fixture
def params(calc, path, time):
    '''Input parameters dict.'''
    return {
        'calculation_type': calc,
        'kernel_paths': path,
        'times': time,
    }

@pytest.fixture
def start():
    '''Input start time.'''
    return '2000-01-01'

@pytest.fixture
def end():
    '''Input end time.'''
    return '2000-01-02'

@pytest.fixture
def interval(start, end):
    '''Input interval dict.'''
    return {"startTime": start, "endTime": end}


def test_calculation_required_err(calc, kernels):
    '''Test error if calculation has missing required attributes.'''
    with pytest.raises(CalculationRequiredAttr):
        Calculation()  # 'calculation_type' missing

    with pytest.raises(CalculationRequiredAttr):
        Calculation(calculation_type=calc)  # 'kernels/kernel_paths' missing

    with pytest.raises(CalculationRequiredAttr):
        Calculation(calculation_type=calc, kernels=kernels)  # 'times' missing

def test_calculation_calc_err(params):
    '''Test error if calculation type is invalid.'''
    params['calculation_type'] = 'WRONG'
    with pytest.raises(CalculationInvalidAttr):
        Calculation(**params)

def test_calculation_time_system_err(params):
    '''Test error if time system is invalid.'''
    params['time_system'] = 'WRONG'
    with pytest.raises(CalculationInvalidAttr):
        Calculation(**params)

def test_calculation_missing_sclk_id(params):
    '''Test error if sclk_id is missing.'''
    params['time_system'] = 'SPACECRAFT_CLOCK'
    with pytest.raises(CalculationUndefinedAttr):
        Calculation(**params)

def test_calculation_time_format_error(params):
    '''Test error if time format is invalid.'''
    params['time_format'] = 'WRONG'
    with pytest.raises(CalculationInvalidAttr):
        Calculation(**params)

def test_calculation_time_system_conflict(params):
    '''Test error when time system and time format are incompatible.'''
    params['time_system'] = 'SPACECRAFT_CLOCK'
    params['sclk_id'] = 1
    with pytest.raises(CalculationIncompatibleAttr):
        Calculation(**params)

def test_calculation_time_format_conflict(params):
    '''Test error when time format and time system are incompatible.'''
    params['time_format'] = 'SPACECRAFT_CLOCK_STRING'
    with pytest.raises(CalculationIncompatibleAttr):
        Calculation(**params)

def test_calculation_sclk_id_conflict(params):
    '''Test error when sclk id system and time system are incompatible.'''
    params['sclk_id'] = 1
    with pytest.raises(CalculationIncompatibleAttr):
        Calculation(**params)

def test_calculation_intervals(params, start, end, interval):
    '''Test calculation interval inputs.'''
    del params['times']
    params['time_step'] = 1
    params['time_step_units'] = 'DAYS'

    params['intervals'] = [start, end]
    assert Calculation(**params).payload['intervals'] == [interval]

    params['intervals'] = {"startTime": start, "endTime": end}
    assert Calculation(**params).payload['intervals'] == [interval]

    params['intervals'] = [interval]
    assert Calculation(**params).payload['intervals'] == [interval]

    params['intervals'] = [interval, interval]
    assert Calculation(**params).payload['intervals'] == [interval, interval]

    params['intervals'] = [interval, interval, interval]
    assert Calculation(**params).payload['intervals'] == [interval, interval, interval]

def test_calculation_intervals_errors(params, start, end, interval):
    '''Test errors if intervals input are incorrect.'''
    params['intervals'] = interval

    with pytest.raises(CalculationConflictAttr):
        Calculation(**params)  # 'times' and 'intervals' conflict

    del params['times']

    with pytest.raises(CalculationUndefinedAttr):
        Calculation(**params)  # 'time_step' missing

    params['time_step'] = 1
    with pytest.raises(CalculationUndefinedAttr):
        Calculation(**params)  # 'time_step_units' missing

    params['time_step_units'] = 'DAYS'

    params['intervals'] = {"startTime": start, "wrongEndTime": end}
    with pytest.raises(CalculationInvalidAttr):
        Calculation(**params)

    params['intervals'] = []
    with pytest.raises(CalculationInvalidAttr):
        Calculation(**params)

    params['intervals'] = [start]
    with pytest.raises(CalculationInvalidAttr):
        Calculation(**params)

    params['intervals'] = [start, interval]
    with pytest.raises(CalculationInvalidAttr):
        Calculation(**params)

    params['intervals'] = [start, end, end]
    with pytest.raises(CalculationInvalidAttr):
        Calculation(**params)

def test_calculation_time_step_conflict(params):
    '''Test error when time step and times are incompatible.'''
    params['time_step'] = 1
    with pytest.raises(CalculationConflictAttr):
        Calculation(**params)

def test_calculation_time_step_units_errors(params, interval):
    '''Test errors if time step units are invalid.'''
    params['time_step_units'] = 'DAYS'
    with pytest.raises(CalculationConflictAttr):
        Calculation(**params)  # Conflict with 'times'

    del params['times']
    params['intervals'] = interval

    with pytest.raises(CalculationUndefinedAttr):
        Calculation(**params)  # 'time_step' missing

    params['time_step'] = 1
    params['time_step_units'] = 'WRONG_UNITS'
    with pytest.raises(CalculationInvalidAttr):
        Calculation(**params)

def test_calculation_aberration_correction_error(params):
    '''Test error if aberration correction is invalid.'''
    with pytest.raises(CalculationInvalidAttr):
        Calculation(aberration_correction='WRONG', **params)

def test_calculation_state_representation_error(params):
    '''Test error if state representation is invalid.'''
    with pytest.raises(CalculationInvalidAttr):
        Calculation(state_representation='WRONG', **params)

def test_calculation_shape_error(params):
    '''Test error if shape 1-2 is invalid.'''
    with pytest.raises(CalculationInvalidAttr):
        Calculation(shape_1='WRONG', **params)

    with pytest.raises(CalculationInvalidAttr):
        Calculation(shape_2='WRONG', **params)

def test_calculation_axis_error(params):
    '''Test error if axis inputs are invalid.'''
    with pytest.raises(CalculationUndefinedAttr):
        Calculation(axis_1='X', **params)  # Missing `orientation_representation`

    with pytest.raises(CalculationIncompatibleAttr):
        # Wrong `orientation_representation` with `axis`
        Calculation(orientation_representation='SPICE_QUATERNION', axis_1='X', **params)

def test_calculation_angular_units_error(params):
    '''Test errors if angular units are invalid.'''
    with pytest.raises(CalculationUndefinedAttr):
        Calculation(angular_units='deg', **params)  # Missing `orientation_representation`

    with pytest.raises(CalculationIncompatibleAttr):
        # Wrong `orientation_representation` with `angular_units`
        Calculation(orientation_representation='SPICE_QUATERNION',
                    angular_units='deg', **params)

def test_calculation_angular_velocity_units_error(params):
    '''Test errors with angular velocity units.'''
    with pytest.raises(CalculationUndefinedAttr):
        # Missing `angular_velocity_representation`
        Calculation(angular_velocity_units='deg/s', **params)

    with pytest.raises(CalculationIncompatibleAttr):
        # Wrong `angular_velocity_representation` with `angular_velocity_units`
        Calculation(angular_velocity_representation='NOT_INCLUDED',
                    angular_velocity_units='deg/s', **params)

def test_calculation_intercept_vector_type_error(params):
    '''Test error if intercept vector type is invalid.'''
    with pytest.raises(CalculationInvalidAttr):
        Calculation(intercept_vector_type='WRONG', **params)

def test_calculation_intercept_instrument_error(params):
    '''Test erros when intercept instrument is invalid.'''
    with pytest.raises(CalculationUndefinedAttr):
        # Missing `intercept_vector_type`
        Calculation(intercept_instrument='CASSINI_ISS_NAC', **params)

    with pytest.raises(CalculationIncompatibleAttr):
        Calculation(intercept_instrument='CASSINI_ISS_NAC',
                    intercept_vector_type='VECTOR_IN_REFERENCE_FRAME',
                    intercept_frame='CASSINI_ISS_NAC', **params)

def test_calculation_intercept_frame_error(params):
    '''Test errors when intercept frame is invalid.'''
    with pytest.raises(CalculationUndefinedAttr):
        # Missing 'intercept_vector_type'
        Calculation(intercept_frame='CASSINI_ISS_NAC', **params)

    with pytest.raises(CalculationIncompatibleAttr):
        # Wrong 'intercept_vector_type'
        Calculation(intercept_frame='CASSINI_ISS_NAC',
                    intercept_vector_type='INSTRUMENT_BORESIGHT', **params)

def test_calculation_intercept_frame_axis_error(params):
    '''Test errors when intercept frame axis is invalid.'''
    with pytest.raises(CalculationUndefinedAttr):
        # Missing 'intercept_frame'
        Calculation(intercept_frame_axis='Z', **params)

    with pytest.raises(CalculationIncompatibleAttr):
        # Wrong 'intercept_vector_type'
        Calculation(intercept_frame_axis='Z',
                    intercept_vector_type='INSTRUMENT_BORESIGHT', **params)

    with pytest.raises(CalculationInvalidAttr):
        # Wrong 'intercept_frame_axis'
        Calculation(intercept_frame_axis='WRONG',
                    intercept_vector_type='REFERENCE_FRAME_AXIS', **params)

def test_calculation_intercept_vector_error(params):
    '''Test errors when intercept vector is invalid.'''
    with pytest.raises(CalculationUndefinedAttr):
        # Missing 'intercept_vector_type'
        Calculation(intercept_vector_x=0, **params)

    with pytest.raises(CalculationIncompatibleAttr):
        # Wrong 'intercept_vector_type'
        Calculation(intercept_vector_x=0,
                    intercept_vector_type='INSTRUMENT_BORESIGHT', **params)

def test_calculation_output_time_system_error(params):
    '''Test errors when output time system is invalid.'''
    with pytest.raises(CalculationInvalidAttr):
        Calculation(output_time_system='WRONG', **params)

    with pytest.raises(CalculationUndefinedAttr):
        # Missing 'output_sclk_id'
        Calculation(output_time_system='SPACECRAFT_CLOCK', **params)

def test_calculation_output_time_format_error(params):
    '''Test errors when output time format is invalid.'''
    with pytest.raises(CalculationInvalidAttr):
        Calculation(output_time_format='WRONG', **params)

    with pytest.raises(CalculationIncompatibleAttr):
        # Wrong 'output_time_system'
        Calculation(output_time_format='CALENDAR',
                    output_time_system='SPACECRAFT_CLOCK', **params)

    with pytest.raises(CalculationIncompatibleAttr):
        # Wrong 'output_time_system'
        Calculation(output_time_format='SPACECRAFT_CLOCK_STRING',
                    output_time_system='UTC', **params)

def test_calculation_output_sclk_id_error(params):
    '''Test errors when output sckl id is invalid.'''
    with pytest.raises(CalculationIncompatibleAttr):
        Calculation(output_sclk_id=-82, output_time_system='WRONG', **params)

def test_calculation_output_time_custom_format_error(params):
    '''Test errors when output time custom is invalid.'''
    with pytest.raises(CalculationIncompatibleAttr):
        Calculation(output_time_custom_format='YYYY Month DD HR:MN',
                    output_time_format='WRONG', **params)



# -*- coding: utf-8 -*-
'''Test WGC time conversion calculation.'''

import pytest

from webgeocalc import TimeConversion

@pytest.fixture
def kernels():
    '''Cassini kernel set.'''
    return 5

@pytest.fixture
def time_1():
    '''Input time in spacecraft clock format.'''
    return '1/1729329441.042'

@pytest.fixture
def time_2():
    '''Input time 2.'''
    return '2012-10-19T08:24:00.000'

@pytest.fixture
def time_system():
    '''Input time system.'''
    return 'SPACECRAFT_CLOCK'

@pytest.fixture
def time_format():
    '''Input time format.'''
    return 'SPACECRAFT_CLOCK_STRING'

@pytest.fixture
def sclk_id():
    '''Cassini spacecraft id.'''
    return -82

@pytest.fixture
def output_time_format():
    '''Input output time format.'''
    return 'CUSTOM'

@pytest.fixture
def output_time_custom_format():
    '''Input output time custom format.'''
    return 'YYYY Month DD HR:MN'

@pytest.fixture
def params_1(kernels, time_1, time_system, time_format, sclk_id):
    '''Input parameters from WGC API example 1.'''
    return {
        'kernels': kernels,
        'times': time_1,
        'time_system': time_system,
        'time_format': time_format,
        'sclk_id': sclk_id,
    }

@pytest.fixture
def payload_1(kernels, time_1, time_system, time_format, sclk_id):
    '''Payload from WGC API example 1.'''
    return {
        "kernels": [{
            "type": "KERNEL_SET",
            "id": kernels,
        }],
        "times": [
            time_1,
        ],
        "calculationType": "TIME_CONVERSION",
        "timeSystem": time_system,
        "timeFormat": time_format,
        "sclkId": sclk_id,
        "outputTimeSystem": "UTC",
        "outputTimeFormat": "CALENDAR",
    }

@pytest.fixture
def params_2(kernels, time_2, output_time_format, output_time_custom_format):
    '''Input parameters from WGC API example 2.'''
    return {
        'kernels': kernels,
        'times': time_2,
        'output_time_format': output_time_format,
        'output_time_custom_format': output_time_custom_format,
    }

@pytest.fixture
def payload_2(kernels, time_2, output_time_format, output_time_custom_format):
    '''Payload from WGC API example 2.'''
    return {
        "kernels": [{
            "type": "KERNEL_SET",
            "id": kernels,
        }],
        "times": [
            time_2,
        ],
        "calculationType": "TIME_CONVERSION",
        "timeSystem": "UTC",
        "timeFormat": "CALENDAR",
        "outputTimeSystem": "UTC",
        "outputTimeFormat": output_time_format,
        "outputTimeCustomFormat": output_time_custom_format,
    }

def test_time_conversion_payload(params_1, payload_1):
    '''Test time conversion payload with spacecraft clock input.'''
    assert TimeConversion(**params_1).payload == payload_1

def test_time_conversion_custom_format_payload(params_2, payload_2):
    '''Test time conversion payload with custom format output.'''
    assert TimeConversion(**params_2).payload == payload_2



'''WebGeoCalc module.'''

import pbr.version

__version__ = pbr.version.VersionInfo('webgeocalc').version_string()

from .api import API
from .calculation import AngularSeparation, AngularSize, Calculation, \
    FrameTransformation, IlluminationAngles, OsculatingElements, StateVector, \
    SubObserverPoint, SubSolarPoint, SurfaceInterceptPoint, TimeConversion

__all__ = [
    'API',
    'AngularSeparation',
    'AngularSize',
    'Calculation',
    'FrameTransformation',
    'IlluminationAngles',
    'OsculatingElements',
    'StateVector',
    'SubObserverPoint',
    'SubSolarPoint',
    'SurfaceInterceptPoint',
    'TimeConversion',
    '__version__',
]



# -*- coding: utf-8 -*-
'''Test WGC command line inputs.'''

from webgeocalc.cli import cli_bodies, cli_frames, cli_instruments, cli_kernel_sets


def test_cli_kernel_sets(capsys):
    '''Test GET kernels with CLI.'''
    argv = ''.split()
    cli_kernel_sets(argv)
    captured = capsys.readouterr()
    assert 'usage:' in captured.out

    argv = '--all'.split()
    cli_kernel_sets(argv)
    captured = capsys.readouterr()
    assert ' - Solar System Kernels: (id: 1)' in captured.out
    assert ' - Cassini Huygens: (id: 5)' in captured.out

    argv = '--kernel 1 Cassini'.split()
    cli_kernel_sets(argv)
    captured = capsys.readouterr()
    assert ' - Solar System Kernels: (id: 1)' in captured.out
    assert 'Too many kernel sets contains \'Cassini\' in their names:' in captured.out

def test_cli_bodies(capsys):
    '''Test GET bodies with CLI.'''
    argv = ''.split()
    cli_bodies(argv)
    captured = capsys.readouterr()
    assert 'usage:' in captured.out

    argv = ['Cassini Huygens']
    cli_bodies(argv)
    captured = capsys.readouterr()
    assert ' - CASSINI: (id: -82)' in captured.out
    assert ' - TITAN: (id: 606)' in captured.out

    argv = '5 --name Titan'.split()
    cli_bodies(argv)
    captured = capsys.readouterr()
    assert ' - TITAN: (id: 606)\n' == captured.out

def test_cli_frames(capsys):
    '''Test GET frames with CLI.'''
    argv = ''.split()
    cli_frames(argv)
    captured = capsys.readouterr()
    assert 'usage:' in captured.out

    argv = ['Cassini Huygens']
    cli_frames(argv)
    captured = capsys.readouterr()
    assert ' - IAU_TITAN: (id: 10044)' in captured.out
    assert ' - J2000: (id: 1)' in captured.out

    argv = '5 --name Titan'.split()
    cli_frames(argv)
    captured = capsys.readouterr()
    assert ' - IAU_TITAN: (id: 10044)' in captured.out
    assert ' - J2000: (id: 1)' not in captured.out

def test_cli_instruments(capsys):
    '''Test GET instruments with CLI.'''
    argv = ''.split()
    cli_instruments(argv)
    captured = capsys.readouterr()
    assert 'usage:' in captured.out

    argv = ['Cassini Huygens']
    cli_instruments(argv)
    captured = capsys.readouterr()
    assert ' - CASSINI_ISS_WAC: (id: -82361)' in captured.out
    assert ' - CASSINI_VIMS_IR: (id: -82370)' in captured.out

    argv = '5 --name ISS'.split()
    cli_instruments(argv)
    captured = capsys.readouterr()
    assert ' - CASSINI_ISS_WAC: (id: -82361)' in captured.out
    assert ' - CASSINI_VIMS_IR: (id: -82370)' not in captured.out
